
#ifndef TESTMACMODULE
#define TESTMACMODULE

#include "VirtualMac.h"
#include "TestMacControl_m.h"
#include "TestMacPacket_m.h"

using namespace std;

enum backoffTypes {
	BACKOFF_SLEEP_INT = 2200,
	BACKOFF_CONSTANT = 2201,
	BACKOFF_MULTIPLYING = 2202,
	BACKOFF_EXPONENTIAL = 2203
};

enum MacStates {
	MAC_STATE_DEFAULT = 1,
	MAC_STATE_TX = 2,
	MAC_STATE_CONTENDING = 3,
	MAC_STATE_RX = 4
};

enum TestMacTimers {
	START_SLEEPING = 1,
	START_LISTENING = 2,
	START_CARRIER_SENSING = 3,
	ATTEMPT_TX = 4,
	SEND_BEACONS_OR_DATA = 5,
};

class TestMAC: public VirtualMac {
 private:
	/*--- The .ned file's parameters ---*/
	double dutyCycle;		// listening interval / sleeping + listening intervals
	double cycleLength;
	bool staticDutyCycle;
	double staticDutyCycleValue;
	
	double listenInterval;	// in secs, note: parammeter in omnetpp.ini in msecs
	double beaconIntervalFraction;
	double probTx;			// probability of a single transmission to happen
	int numTx;				// when we have something to send, how many times do we try to transmit it.
							// We say "try" because probTx might be < 1

	double randomTxOffset;	// when have something to transmit, don't do it immediatelly
	double reTxInterval;	// the interval between retransmissions, in msec but after a
							// time [0..randomTxOffset] chosen randomly (uniform)
	int beaconFrameSize;	// in bytes
	int backoffType;		// can be 0 or 1 or 2 or 3
	double backoffBaseValue;// the backoff value
	double CSMApersistance; // value in [0..1], is CSMA non-persistent, p-persistent, or 1-persistent
	bool txAllPacketsInFreeChannel; // when channel free, tx all packets in buffer?
	bool sleepDuringBackoff;	// for no dutyCycle case: sleep when backing off?

	int phyLayerOverhead;
	simtime_t phyDelayForValidCS;	// delay for valid CS
	double phyDataRate;

	int macState;
	int numTxTries;
	int backoffTimes;	// number of consequtive backoff times
	int remainingBeaconsToTx;
	double beaconTxTime;	// time it takes to TX a beacon in secs
	double sleepInterval;	// in secs

 protected:
	void startup();
	void fromNetworkLayer(cPacket *, int);
	void fromRadioLayer(cPacket *, double, double);
	int handleControlCommand(cMessage * msg);
	void timerFiredCallback(int);
	void handleCarrierSenseResult(int);
	void attemptTx();
	void sendBeaconsOrData();
	double calcDutyCycle(double r0, int maxDefinedCoverSetNumber, double minDutyCycle, double maxDutyCycle, int sizeofCoverset);
};

#endif				//TESTMACMODULE
