#ifndef __TEESTREAM_H
#define __TEESTREAM_H

#include <streambuf>
#include <iostream>

class teebuf: public std::streambuf
{
public:
    // Construct a streambuf which tees output to both input
    // streambufs.
    teebuf() { }
    teebuf(std::streambuf * sb1, std::streambuf * sb2)
        : sb1(sb1)
        , sb2(sb2)
    {
    }
    
    void set_streambuf(std::streambuf * the_sb1, std::streambuf * the_sb2) {
    	sb1=the_sb1;
    	sb2=the_sb2;
    }
    
private:
    // This tee buffer has no buffer. So every character "overflows"
    // and can be put directly into the teed buffers.
    virtual int overflow(int c)
    {
        if (c == EOF)
        {
            return !EOF;
        }
        else
        {
            int const r1 = sb1->sputc(c);
            int const r2 = sb2->sputc(c);
            return r1 == EOF || r2 == EOF ? EOF : c;
        }
    }
    
    // Sync both teed buffers.
    virtual int sync()
    {
        int const r1 = sb1->pubsync();
        int const r2 = sb2->pubsync();
        return r1 == 0 && r2 == 0 ? 0 : -1;
    }   
private:
    std::streambuf * sb1;
    std::streambuf * sb2;
};

class teestream : public std::ostream
{
public:
    // Construct an ostream which tees output to the supplied
    // ostreams.
   	teestream() : std::ostream(&tbuf) { }
    
	teestream(std::ostream & o1, std::ostream & o2)
  : std::ostream(&tbuf)
  , tbuf(o1.rdbuf(), o2.rdbuf()) { }
  
  	void set_streambuf(std::ostream & o1, std::ostream & o2) {
  		tbuf.set_streambuf(o1.rdbuf(), o2.rdbuf());
  	 } 

private:
    teebuf tbuf;
};


#endif


