void VideoSensorNode::handle2hopDataDiffusion() {

    if (this->nbNeighbors == 0){
        //ev << "Node " << this->id << "has no neighbor. It can't diffuse 2-hop information" << endl;
        return;
    }

    TwoHopInfoPacket *pkt = new TwoHopInfoPacket("2-hop data diffusion", APPLICATION_PACKET);
    pkt->setNodeId(this->id);
    pkt->setSpeed(this->getCaptureRate());
    pkt->setEnergy(this->energyLevel);
    pkt->setIdListArraySize(this->nbNeighbors);
    for (unsigned int j = 0; j < this->nbNeighbors; j++)
        pkt->setIdList(j, this->neighbors[j].id);

    ev << "Node " << this->id << " --> 2-hop data diffusion to its " << this->nbNeighbors <<  " neighbors " << endl;
    trace() << "Node " << this->id << " --> 2-hop data diffusion to its " << this->nbNeighbors <<  " neighbors " << endl;
    // now send it to the network layer
    toNetworkLayer(pkt, BROADCAST_NETWORK_ADDRESS);

    scheduleAt(simTime()+5.0, this->startComputeScore);
    scheduleAt(simTime() + 1.0, this->startCoverMemberDiffusion);

}

void VideoSensorNode::handleCoverMemberDiffusionInitiate() {

    if (this->nbNeighbors == 0){
       //ev << "Node " << this->id << " has no neighbor. It can't diffuse cover member information" << endl;
       return;
    }

    if (this->nodeCoverset.size() == 0){
       //ev << "Node " << this->id << " has no cover set. It can't diffuse cover member information" << endl;
       return;
    }

    CoverMemberPacket  *pkt = new CoverMemberPacket("start cover member diffusion", APPLICATION_PACKET);
    pkt->setNodeId(this->id);
    // destinations list : cover members of current node
    pkt->setIdListArraySize(0);
    int j = 0;
    for (unsigned int i = 0; i < this->nbNeighbors; i++)
        if (this->isCoverMember(this->neighbors[i].id)){
            pkt->setIdListArraySize(pkt->getIdListArraySize() + 1);
            pkt->setIdList(j, this->neighbors[i].id);
            j++;
        }
     ev << "Node " << this->id << " --> start cover member diffusion to nodes (";
     for (unsigned int k = 0; k < pkt->getIdListArraySize(); k++){
        ev << pkt->getIdList(k);
        if ( k < pkt->getIdListArraySize()-1)
            ev << ", ";
     }
     ev << ")" << endl;
     // now send it to the network layer
     toNetworkLayer(pkt, BROADCAST_NETWORK_ADDRESS);

}

void VideoSensorNode::handleComptuteScore() {

    if (this->nbNeighbors == 0){
        //ev << "Noeud " << this->id << " has no neighbor" << endl;
        return;
    }
    showNeighbors();

    if (this->nodeCoverset.size() == 0){
         //ev << "Noeud " << this->id << " has no cover set" << endl;
         return;
    }

    //ev << "Node " << this->id << " initiates compute score for the Sink node " << this->recipientAddress << endl;
    //trace() << "Node " << this->id << " initiates compute score for the Sink node " << this->recipientAddress << endl;

    // we need the nextRecipient's position (Sink's position)
    noeud* nextRecipientInfo = CoordNode->getNodeInfo(atoi(this->recipientAddress.c_str()));
    if (nextRecipientInfo) {
        //showNeighbors();
        setCompleteCover();
        setCoversetsEnergy();
        setDegreeOfJoint();
        setWeightSize();
        setWeightDependency();
        setOtherWeights(nextRecipientInfo->p.x, nextRecipientInfo->p.y);
        setScore();
        showCoversets();
        ev << "\tBest cover set index : " << getCoversetIndex() << endl;
        trace() << "Best cover set index : " << getCoversetIndex() << endl;
        ev << endl;
    }
    else{
        ev << "Node " << this->id << ": cannot find sink's information.";
        trace() << "Node " << this->id << ": cannot find sink's information.";
    }

}

void VideoSensorNode::handle2hopDataDiffusionPacket(cMessage* msg) {

    TwoHopInfoPacket *rcvPacket = check_and_cast<TwoHopInfoPacket *>(msg);
    int nIndex = getNeighborIndexById(rcvPacket->getNodeId());
    if (nIndex != -1){
        this->neighbors[nIndex].speed = rcvPacket->getSpeed();
        this->neighbors[nIndex].Energy = rcvPacket->getEnergy();
        int index = 0;
        for (unsigned int k = 0; k < rcvPacket->getIdListArraySize(); k++){
            int tok = rcvPacket->getIdList(k);
            if ((tok != this->id) && (this->getNeighborIndexById(tok) == -1)){
                noeud* nodeInfo = CoordNode->getNodeInfo(tok);
                if (nodeInfo) {
                    this->neighbors[nIndex].myNeighbors[index].id = tok;
                    this->neighbors[nIndex].myNeighbors[index].p = Point(nodeInfo->p.x, nodeInfo->p.y);
                    index++;
                 }
                 else{
                    ev << "Node " << this->id << " : cannot find node " << tok << "'s information" << endl;
                    trace() << "Node " << this->id << ": cannot find node " << tok << "'s information" << endl;
                 }
            }
        }
        this->neighbors[nIndex].numNeighbors = index;
        if (this->neighbors[nIndex].numNeighbors > 0)
            this->neighbors[nIndex].notEmptyNeighborSet = true;
    }

}

void VideoSensorNode::handleStartCoverMemberDiffusionPacket(cMessage* msg) {

    CoverMemberPacket  *rcvPacket = check_and_cast<CoverMemberPacket *>(msg);
    // Is it a good destination?
    unsigned int k = 0;
    while ((k < rcvPacket->getIdListArraySize()) && (rcvPacket->getIdList(k) != this->id))
        k++;
    if (k >= rcvPacket->getIdListArraySize()) // it isn't a good destination
        return;

    ev << "Node " << this->id << " <-- start cover member diffusion from node " << rcvPacket->getNodeId() << endl;
    CoverMemberPacket  *pkt = new CoverMemberPacket("cover member diffusion", APPLICATION_PACKET);
    pkt->setNodeId(this->id);
    pkt->setIdListArraySize(0);
    pkt->setNumNeighborIdListArraySize(0);
    pkt->setNeighborIdListArraySize(0);
    pkt->setDest(rcvPacket->getNodeId());
    int index = 0;  unsigned int nb = 0;
    for (unsigned int i = 0; i < this->nbNeighbors; i++){
        if((this->neighbors[i].id != rcvPacket->getNodeId()) && (this->neighbors[i].numNeighbors > 0)){
            unsigned int t = 0;
            while ((t < rcvPacket->getIdListArraySize()) && (rcvPacket->getIdList(t) != this->neighbors[i].id))
                t++;
             if (t >= rcvPacket->getIdListArraySize()){
                pkt->setIdListArraySize(pkt->getIdListArraySize() + 1);
                pkt->setIdList(nb, this->neighbors[i].id);
                pkt->setNumNeighborIdListArraySize(pkt->getNumNeighborIdListArraySize() + 1);
                pkt->setNumNeighborIdList(nb, this->neighbors[i].numNeighbors);
                nb++;
                pkt->setNeighborIdListArraySize(pkt->getNeighborIdListArraySize() + this->neighbors[i].numNeighbors);
                for (int j = 0; j < this->neighbors[i].numNeighbors; j++){
                    pkt->setNeighborIdList(index,this->neighbors[i].myNeighbors[j].id);
                    index++;
                }
             }
        }
    }

    // now send it to the network layer
    if (pkt->getIdListArraySize() > 0){
        ev << "Node " << this->id << " --> cover member diffusion to node " << pkt->getDest();
        ev << " with (";
        int index1 = 0;
        for (unsigned int z = 0; z < pkt->getIdListArraySize(); z++){
            ev << pkt->getIdList(z);
            if (pkt->getNumNeighborIdList(z) > 0){
                ev << " [ ";
                for (int y = 0; y < pkt->getNumNeighborIdList(z); y++){
                    ev << pkt->getNeighborIdList(index1);
                    if ( y < pkt->getNumNeighborIdList(z)-1)
                        ev << ", ";
                    index1++;
                }
                ev << " ]";
             }
             if ( z < pkt->getIdListArraySize()-1)
                ev << ", ";
        }
        ev << ")" << endl;
        toNetworkLayer(pkt, BROADCAST_NETWORK_ADDRESS);
    }
    else
        ev << "Node " << this->id << " can't send ""cover member diffusion"" msg to node " << pkt->getDest() << endl;

}

void VideoSensorNode::handleCoverMemberDiffusionPacket(cMessage* msg) {

    CoverMemberPacket  *rcvPacket = check_and_cast<CoverMemberPacket *>(msg);

    if (rcvPacket->getDest() != this->id){
        //ev << "No good destination" << endl;
        return;
    }
    ev << "Node " << this->id << " <-- cover member diffusion from node " << rcvPacket->getNodeId() << endl;
    int nIndex = getNeighborIndexById(rcvPacket->getNodeId());
    if (nIndex != -1){
    if(this->neighbors[nIndex].numNeighbors == 0){
        this->neighbors[nIndex].myNeighbors = new noeud[rcvPacket->getIdListArraySize()];
        this->neighbors[nIndex].numNeighbors = rcvPacket->getIdListArraySize();
        int nIndex1 = 0; int index = 0;
        for (unsigned int k = 0; k < rcvPacket->getIdListArraySize(); k++){
            this->neighbors[nIndex].myNeighbors[nIndex1].id = rcvPacket->getIdList(k);
            this->neighbors[nIndex].myNeighbors[nIndex1].myNeighbors = new noeud[rcvPacket->getNumNeighborIdList(k)];
            this->neighbors[nIndex].myNeighbors[nIndex1].numNeighbors = rcvPacket->getNumNeighborIdList(k);
            for (int j = 0; j < rcvPacket->getNumNeighborIdList(k); j++){
                noeud* nodeInfo1 = CoordNode->getNodeInfo(rcvPacket->getNeighborIdList(index));
                if (nodeInfo1){
                    this->neighbors[nIndex].myNeighbors[nIndex1].myNeighbors[j].id = rcvPacket->getNeighborIdList(index);
                    this->neighbors[nIndex].myNeighbors[nIndex1].myNeighbors[j].p = Point(nodeInfo1->p.x, nodeInfo1->p.y);
                    index++;
                }
                else{
                    ev << "Node " << this->id << " : cannot find node " << rcvPacket->getNeighborIdList(index) << "'s information" << endl;
                    trace() << "Node " << this->id << " : cannot find node " << rcvPacket->getNeighborIdList(index) << "'s information" << endl;
                }
            }
            nIndex1++;
        }
    }

    else { // this->neighbors[nIndex].numNeighbors > 0
        int index1 = 0;
        for (unsigned int k = 0; k < rcvPacket->getIdListArraySize(); k++){
            int nIndex2 = getMyNeighborIndexById(nIndex, rcvPacket->getIdList(k));
            if (nIndex2 != -1){
                this->neighbors[nIndex].myNeighbors[nIndex2].myNeighbors = new noeud[rcvPacket->getNumNeighborIdList(k)];
                this->neighbors[nIndex].myNeighbors[nIndex2].numNeighbors = rcvPacket->getNumNeighborIdList(k);
                for (int j = 0; j < rcvPacket->getNumNeighborIdList(k); j++){
                    noeud* nodeInfo = CoordNode->getNodeInfo(rcvPacket->getNeighborIdList(index1));
                    if (nodeInfo){
                        this->neighbors[nIndex].myNeighbors[nIndex2].myNeighbors[j].id = rcvPacket->getNeighborIdList(index1);
                        this->neighbors[nIndex].myNeighbors[nIndex2].myNeighbors[j].p = Point(nodeInfo->p.x, nodeInfo->p.y);
                        index1++;
                    }
                    else{
                        ev << "Node " << this->id << " : cannot find node " << rcvPacket->getNeighborIdList(index1) << "'s information" << endl;
                        trace() << "Node " << this->id << " : cannot find node " << rcvPacket->getNeighborIdList(index1) << "'s information" << endl;
                    }
                }
            }

            else{
                int n = this->neighbors[nIndex].numNeighbors;
                this->neighbors[nIndex].numNeighbors++;
                this->neighbors[nIndex].myNeighbors[n].numNeighbors = rcvPacket->getNumNeighborIdList(k);
                this->neighbors[nIndex].myNeighbors[n].myNeighbors = new noeud[rcvPacket->getNumNeighborIdList(k)];
                for (int j = 0; j < rcvPacket->getNumNeighborIdList(k); j++){
                    noeud* nodeInfo = CoordNode->getNodeInfo(rcvPacket->getNeighborIdList(index1));
                    if (nodeInfo){
                        this->neighbors[nIndex].myNeighbors[n].myNeighbors[j].id = rcvPacket->getNeighborIdList(index1);
                        this->neighbors[nIndex].myNeighbors[n].myNeighbors[j].p = Point(nodeInfo->p.x, nodeInfo->p.y);
                        index1++;
                    }
                    else{
                        ev << "Node " << this->id << " : cannot find node " << rcvPacket->getNeighborIdList(index1) << "'s information" << endl;
                        trace() << "Node " << this->id << " : cannot find node " << rcvPacket->getNeighborIdList(index1) << "'s information" << endl;
                    }
                }
            }
                }
        }
    }

}

int VideoSensorNode::getMyNeighborIndexById(int NeighborIndex, int theId) {

    for (int i=0; i<this->neighbors[NeighborIndex].numNeighbors; i++)
        if (theId == this->neighbors[NeighborIndex].myNeighbors[i].id)
            return i;
    return -1;
}

bool coversetElem::belong(int _id) {

    SetInt myset;
    myset.insert(_id);
    return includes(aCoverset.begin(),aCoverset.end(),myset.begin(),myset.end());

}

bool VideoSensorNode::isCoverMember(int _id){

    for (unsigned int i = 0; i < nodeCoverset.size(); i++)
        if (coversetTable[i].belong(_id))
            return true;
    return false;

}

void VideoSensorNode::showNeighbors(){

    if (this->nbNeighbors == 0){
        ev << "Node " << this->id << " has no neighbor" << endl;
        trace() << "Node " << this->id << " has no neighbor" << endl;
        return;
    }

    ev << "Node " << this->id << ": has " << this->nbNeighbors <<" neighbors ";
    ev << "\t( ";
    for (unsigned int i = 0; i < this->nbNeighbors; i++){
        ev << this->neighbors[i].id << " ";
        if (this->neighbors[i].numNeighbors > 0){
            ev << "[ ";
            for (int s = 0; s < this->neighbors[i].numNeighbors; s++){
               ev << this->neighbors[i].myNeighbors[s].id;
               if(this->isCoverMember(this->neighbors[i].id) && (this->neighbors[i].myNeighbors[s].numNeighbors > 0)){
                   ev << " { ";
                   for (int j = 0; j < this->neighbors[i].myNeighbors[s].numNeighbors; j++){
                       ev << this->neighbors[i].myNeighbors[s].myNeighbors[j].id;
                       if (s < this->neighbors[i].myNeighbors[s].numNeighbors - 1)
                           ev << ", ";
                    }
                    ev << " } ";
               }
               if (s < this->neighbors[i].numNeighbors - 1)
                   ev << ", ";
            }
            ev << " ] ";
        }
        if (i < this->nbNeighbors - 1)
           ev << ", ";
    }
    ev << ")" << endl;

}

void VideoSensorNode::showCoversets(){

    if (this->nodeCoverset.size() == 0){
        ev << "Node " << this->id << " has no cover set" << endl;
        trace() << "Node " << this->id << " has no cover set" << endl;
        return;
    }

    ev << "Node " << this->id << " : has " << this->nodeCoverset.size() << " cover sets " << endl;
    ev << "\tIndex \tMembers \tWSize \tWJoint \tWAv1HopNg \tWCAv1HopNg \tWAv2HopNg \tWCAv2HopNg \tScore" << endl;
    for (unsigned int i = 0; i < nodeCoverset.size(); i++) {
        ev << "\t" << i;
        ev << "\t(";
        for (int j = 0; j <  coversetTable[i].coversetSize; j++){
            ev << coversetTable[i].aCompleteCoverset[j].id;
            if(j < coversetTable[i].coversetSize - 1)
                ev << ", ";
        }
        ev << ")";
        ev << "\t" << coversetTable[i].weightSize << "\t" << coversetTable[i].weightDependency << "\t" << coversetTable[i].weightAv1HopNeighbors << "\t\t" << coversetTable[i].weightClosestAv1HopNeighbors
           << "\t\t" << coversetTable[i].weightAv2HopNeighbors << "\t\t" << coversetTable[i].weightClosestAv2HopNeighbors << "\t\t" << coversetTable[i].score << endl;
    }

}


void VideoSensorNode::setCompleteCover(){

    for (unsigned int i = 0; i < nodeCoverset.size(); i++) {
        // initialize aCompleteCoverset table
        coversetTable[i].aCompleteCoverset = new elem[coversetTable[i].coversetSize];
        int j = 0;
        SetInt::iterator x;
        for(x=coversetTable[i].aCoverset.begin(); x!=coversetTable[i].aCoverset.end(); x++){
            int k = getNeighborIndexById(*x);
            coversetTable[i].aCompleteCoverset[j].id = *x;
            coversetTable[i].aCompleteCoverset[j].speed = this->neighbors[k].speed;
            coversetTable[i].aCompleteCoverset[j].p  =  Point(this->neighbors[k].p.x,this->neighbors[k].p.y);

            if(this->neighbors[k].numNeighbors > 0){
                coversetTable[i].aCompleteCoverset[j].listNeighborsSize = this->neighbors[k].numNeighbors;
                coversetTable[i].aCompleteCoverset[j].listNeighbors = new noeud[this->neighbors[k].numNeighbors];

                for (int z = 0; z < this->neighbors[k].numNeighbors; z++){
                    coversetTable[i].aCompleteCoverset[j].listNeighbors[z].id = this->neighbors[k].myNeighbors[z].id;
                    coversetTable[i].aCompleteCoverset[j].listNeighbors[z].speed = this->neighbors[k].myNeighbors[z].speed;
                    coversetTable[i].aCompleteCoverset[j].listNeighbors[z].p  =  Point(this->neighbors[k].myNeighbors[z].p.x,this->neighbors[k].myNeighbors[z].p.y);

                    if(this->neighbors[k].myNeighbors[z].numNeighbors > 0){
                        coversetTable[i].aCompleteCoverset[j].listNeighbors[z].numNeighbors = this->neighbors[k].myNeighbors[z].numNeighbors;
                        coversetTable[i].aCompleteCoverset[j].listNeighbors[z].myNeighbors = new noeud[this->neighbors[k].myNeighbors[z].numNeighbors];

                        for (int t = 0; t < this->neighbors[k].myNeighbors[z].numNeighbors;t++){
                           coversetTable[i].aCompleteCoverset[j].listNeighbors[z].myNeighbors[t].id = this->neighbors[k].myNeighbors[z].myNeighbors[t].id;
                           coversetTable[i].aCompleteCoverset[j].listNeighbors[z].myNeighbors[t].p = Point(this->neighbors[k].myNeighbors[z].myNeighbors[t].p.x,this->neighbors[k].myNeighbors[z].myNeighbors[t].p.y);
                        }
                    }
                }
            }

            j++;
        }       
    }

}

int VideoSensorNode::getNeighborEnergy(int _id){

    for (unsigned int i = 0; i < nbNeighbors; i++)
        if(neighbors[i].id == _id)
            return neighbors[i].Energy;
    return -1;

}

void VideoSensorNode::setCoversetsEnergy(){

    for (unsigned int i = 0; i < nodeCoverset.size(); i++){
        int _id = *(coversetTable[i].aCoverset.begin()); //to recover the id of the 1st element
        int minEnergy = getNeighborEnergy(_id); //to recover his energy
        SetInt::iterator x;
        for(x=coversetTable[i].aCoverset.begin(); x!=coversetTable[i].aCoverset.end(); x++)
            if(minEnergy > getNeighborEnergy(*x))
                minEnergy = getNeighborEnergy(*x);
        coversetTable[i].energy = minEnergy;
    }
}

void VideoSensorNode::setDegreeOfJoint(){

    for (unsigned int i = 0; i < nodeCoverset.size(); i++){
        // treate coversetTable[i]
        coversetTable[i].degreeOfJoint = 0;
        SetInt::iterator x;
        for(x=coversetTable[i].aCoverset.begin(); x!=coversetTable[i].aCoverset.end(); x++){
            unsigned int k = 0;    bool stop = false;
            while( (k < nodeCoverset.size()) && (! stop) ){
                if (i != k) {
                    if (coversetTable[k].belong(*x)){
                        coversetTable[i].degreeOfJoint++;
                        stop = true;
                    }
                }
                k++;
            }
        }
    }

}

void VideoSensorNode::setWeightSize() {

    for (unsigned int i = 0; i < nodeCoverset.size(); i++) {
        if (coversetTable[i].coversetSize == 2)
            coversetTable[i].weightSize = 0.5; // or 1
        else // coversetTable[i].coversetSize=3
            coversetTable[i].weightSize = 0.25; // or O.5
    }

}

void VideoSensorNode::setWeightDependency() {

    for (unsigned int i = 0; i < nodeCoverset.size(); i++) {
        switch (coversetTable[i].degreeOfJoint) {
            case 0:{ coversetTable[i].weightDependency = 1;
                break;
            }
            case 1:{ coversetTable[i].weightDependency = 0.75;
                break;
            }
            case 2:{ coversetTable[i].weightDependency = 0.5;
                break;
            }
            case 3:{ coversetTable[i].weightDependency = 0.25;
                break;
            }
                default: break;
        }
    }

}

void VideoSensorNode::setOtherWeights(double xSink, double ySink) {

    Point pSink(xSink, ySink);
    for (unsigned int i = 0; i < nodeCoverset.size(); i++){
        coversetTable[i].weightAv1HopNeighbors = 0;
        coversetTable[i].weightClosestAv1HopNeighbors = 0;
        coversetTable[i].weightAv2HopNeighbors = 0;
        coversetTable[i].weightClosestAv2HopNeighbors = 0;
        coversetTable[i].weightDistance = 0;
        double som1, som2, som3, som4, som5;
        som1 = 0; som2 = 0; som3 = 0; som4 = 0; som5 = 0;

        for (int j = 0; j < coversetTable[i].coversetSize; j++){
            // compute the distance of cover
            som3 = som3 + distance(coversetTable[i].aCompleteCoverset[j].p, pSink);
            double myDistToSink = distance(coversetTable[i].aCompleteCoverset[j].p, pSink);
            int count = 0;
            if(coversetTable[i].aCompleteCoverset[j].listNeighborsSize > 0){
                for (int k = 0; k < coversetTable[i].aCompleteCoverset[j].listNeighborsSize; k++){
                    // compute number of closest available 1-hop neighbors
                    double dist = distance(coversetTable[i].aCompleteCoverset[j].listNeighbors[k].p, pSink);
                    if (dist < myDistToSink)
                        count++;
                    // closest 2 hop available neighbors
                    if(coversetTable[i].aCompleteCoverset[j].listNeighbors[k].numNeighbors > 0){
                        // compute number of available 2-hop neighbors
                        som4 = som4 + coversetTable[i].aCompleteCoverset[j].listNeighbors[k].numNeighbors;
                        // closest 2 hop available neighbors
                        int count1 = 0;
                        double myNeighDistToSink = distance(coversetTable[i].aCompleteCoverset[j].listNeighbors[k].p, pSink);
                        for (int t = 0; t < coversetTable[i].aCompleteCoverset[j].listNeighbors[k].numNeighbors; t++){
                            // compute number of closest available 2-hop neighbors
                            double dist = distance(coversetTable[i].aCompleteCoverset[j].listNeighbors[k].myNeighbors[t].p, pSink);
                            if (dist < myNeighDistToSink)
                                count1++;
                        } // end for t
                        som5 = som5 + count1;
                    }
                } // end for k
            }
            if(floor(coversetTable[i].aCompleteCoverset[j].speed) == 0){ // ==> speed = 0 + 1
              som1 += som1 + coversetTable[i].aCompleteCoverset[j].listNeighborsSize; // available 1-hop neighbors
              som2 += som2 + count; // closest available 1-hop neighbors
            }
            else{
              som1 += som1 + (coversetTable[i].aCompleteCoverset[j].listNeighborsSize / floor (coversetTable[i].aCompleteCoverset[j].speed));
              som2 += som2 + (count / floor (coversetTable[i].aCompleteCoverset[j].speed));
            }
        } // end for j
        coversetTable[i].weightAv1HopNeighbors = som1;
        coversetTable[i].weightClosestAv1HopNeighbors = som2;
        coversetTable[i].weightAv2HopNeighbors = som4;
        coversetTable[i].weightClosestAv2HopNeighbors = som5;
        if (som3 > 0)
            coversetTable[i].weightDistance = 100/som3; // normalize : minimum_distance/cover_distance
    } // end for i

}

void VideoSensorNode::setScore() {

    for (unsigned int i = 0; i < nodeCoverset.size(); i++){
        coversetElem cover = coversetTable[i];
        coversetTable[i].score = 0;
        coversetTable[i].score = (cover.weightSize + cover.weightDependency + cover.weightAv1HopNeighbors +
        cover.weightClosestAv1HopNeighbors + cover.weightAv2HopNeighbors + cover.weightClosestAv2HopNeighbors)/6;
    }

}

int VideoSensorNode::bestCoversetSelection() {

    int coversetIndex = -1;
    unsigned int i = 0;
    while ((i < nodeCoverset.size()) && (coversetTable[i].energy < THRESHOLD_ENERGY))
        i++;
    if (i < nodeCoverset.size()){
        double maxScore = coversetTable[i].score;
        coversetIndex = i;
        for (unsigned int j = i+1; j < nodeCoverset.size(); j++){
            if ((coversetTable[j].score > maxScore) && (coversetTable[j].energy >= THRESHOLD_ENERGY)){
                maxScore = coversetTable[j].score;
                coversetIndex = j;
            }
            // equal score : choice the greatest energy (simple method)
            else if ((coversetTable[j].score == maxScore) && (coversetTable[j].energy >= 10)){
                if (coversetTable[j].energy > coversetTable[i].energy){
                    maxScore = coversetTable[j].score;
                    coversetIndex = j;
                }
            }
        }
    }
    return coversetIndex;

}
