// mainly taken from the connectivityMap_ApplicationModule from Castalia
// C. Pham, Congduc.Pham@univ-pau.fr, www.univ-pau.fr/~cpham 

#include "connectivityMap.h"
#include "videoSensorNode.h"
#include "coordNode.h"

#ifdef CASTALIA
void connectivityMap::init(int _s, int numNodes, double _x, double _y, Radio *theRadio)
#else
void connectivityMap::init(int _s, int numNodes, double _x, double _y)
#endif
{
	self = _s;
	self_xCoo = _x;
	self_yCoo = _y;
	neighborTable.clear();
	neighborTable.reserve(numNodes);

#ifdef CASTALIA
	radioModule = theRadio;
#ifdef CASTALIA_MODIFIED_RADIO
	initialTxOutputPower = radioModule->getCurrentTxOutputPower();
#else
	string startingTxPower = radioModule->par("TxOutputPower");
	char *tmp;
	initialTxOutputPower = strtod(startingTxPower.c_str(), &tmp);
#endif

	currentTxOutputPower = initialTxOutputPower;
	nbTxOutputPowerLevels = 1;

#if defined(CASTALIA_CHANGE_POWER_LEVELS) && defined(CASTALIA_MODIFIED_RADIO)
	nbTxOutputPowerLevels = radioModule->getNbTxOutputLevel();
#endif
#endif

	packetsSent = 0;
	serialNumber = 0;
	totalSNnodes=numNodes; 

	txInterval_perNode = SENT_PACKETS_PER_NODE_PER_LEVEL * INTER_PACKET_SPACING;
	txInterval_perOutputPowerLevel = txInterval_perNode * totalSNnodes;
}

void connectivityMap::updateNeighborTable(int nodeID, int theSN, int px, int py, int vx, int vy, double dov, double alpha)
{
	int i=0, pos = -1;
	int tblSize = (int)neighborTable.size();
	double myDistance;

	for (i=0; i < tblSize; i++)
		if (neighborTable[i].id == nodeID)
			pos = i;

	if (pos == -1) {
		vid_neighborRecord newRec, newRec1;
		newRec.id = nodeID;
		newRec.timesRx = 1;
		// stores the x,y coordinate here
		newRec.px=px;
		newRec.py=py;
		newRec.vx=vx;
		newRec.vy=vy;
		newRec.dov=dov;
		newRec.alpha=alpha;
		myDistance = sqrt((self_xCoo - newRec.px)*(self_xCoo - newRec.px) + 
						(self_yCoo - newRec.py)*(self_yCoo - newRec.py))/(MAXFIELDUNIT/MAXFIELDMETER);									
		newRec.theDistance=myDistance;
		
		if ( (theSN >= 0) && (theSN < SENT_PACKETS_PER_NODE_PER_LEVEL) )
			newRec.receivedPackets = 1;										

		neighborTable.push_back(newRec);

#ifdef DEBUG_OUTPUT_LEVEL3
		tblSize=(int)neighborTable.size();
		pos=tblSize-1;
		// print the last item
		cout << "id:" << neighborTable[pos].id << " px:" << neighborTable[pos].px << " py:" << neighborTable[pos].py << 
				" vx:" << neighborTable[pos].vx << " vy:" << neighborTable[pos].vy << " alpha:" << neighborTable[pos].alpha << 
				" D:" << neighborTable[pos].theDistance << endl;
		cout << "dumb1:" << neighborTable[pos].dumb1 << " dumb2:" << neighborTable[pos].dumb2 << endl;
#endif
	}
	else {
		neighborTable[pos].timesRx++;
		myDistance=neighborTable[pos].theDistance;

		if ( (theSN >= 0) && (theSN < SENT_PACKETS_PER_NODE_PER_LEVEL) )
			neighborTable[pos].receivedPackets++;
	}
	
#ifdef DEBUG_OUTPUT_LEVEL2
		ev << "Node " << self << " (" << (int)self_xCoo << "," << (int)self_yCoo << ") <-- Node " << nodeID << " (" << px << "," << py << 
			") SN=" << theSN << " pos=" << pos << " gives " << myDistance << "m" << endl;
#endif	
}

