// mainly taken from the connectivityMap_ApplicationModule from Castalia
// C. Pham, Congduc.Pham@univ-pau.fr, www.univ-pau.fr/~cpham 

#ifndef _CONNECTIVITYMAP_H_
#define _CONNECTIVITYMAP_H_

#include <string>
#include <vector>
#include <omnetpp.h>
#include "videoModelDefine.h"

#ifdef CASTALIA
#include "Radio.h"
#endif

using namespace std;


/************ Connectivity Map Definitions  ******************************/

#define INTER_PACKET_SPACING 0.01
#define SENT_PACKETS_PER_NODE_PER_LEVEL 3

/************ End of Connectivity Map Definitions  ******************************/

struct vid_neighborRecord
{
	int id;
	float dov;
	float alpha;
	float theDistance;
	int timesRx;
	int receivedPackets;
	// modified to add the coordinate for distance calculation
	int px;
	int py;
	int vx;
	int vy;

	vid_neighborRecord() {
		id=-1;
		alpha=0.0;
		dov=0.0;
		theDistance=0.0;
		timesRx=0;
		receivedPackets=0;
		px=0;
		py=0;
		vx=0;
		vy=0;	
	}	
	// copy constructor
	vid_neighborRecord (const vid_neighborRecord &n) {
		id=n.id;
		alpha=n.alpha;
		dov=n.dov;
		theDistance=n.theDistance;
		timesRx=n.timesRx;
		receivedPackets=n.receivedPackets;
		px=n.px;
		py=n.py;
		vx=n.vx;
		vy=n.vy;	
	}	

	// overload = operator
	vid_neighborRecord& operator= (const vid_neighborRecord &n) {
		id=n.id;
		alpha=n.alpha;
		dov=n.dov;
		theDistance=n.theDistance;
		timesRx=n.timesRx;
		receivedPackets=n.receivedPackets;
		px=n.px;
		py=n.py;
		vx=n.vx;
		vy=n.vy;	
		return *this;
	}	
};


class connectivityMap
{
public:
        
	int self;	// the node's ID
	// stores the node's position
	double self_xCoo;
	double self_yCoo;
#ifdef CASTALIA
	Radio *radioModule;	//a pointer to the radio module object
#endif

	vector <vid_neighborRecord> neighborTable;
	int packetsSent;
	int serialNumber;
	int totalSNnodes;
	int nbTxOutputPowerLevels;
	int currentTxOutputPowerIndex;
	double txInterval_perNode;
	double txInterval_perOutputPowerLevel;
	double currentTxOutputPower;
	double initialTxOutputPower;

#ifdef CASTALIA
	void init(int, int, double, double, Radio*);
#else
	void init(int, int, double, double);
#endif
	// adds the x,y coordinate in the neighbor update process
	void updateNeighborTable(int, int, int, int, int, int, double, double);
};

#endif // _CONNECTIVITYMAP_H_
