// C. Pham, Congduc.Pham@univ-pau.fr, www.univ-pau.fr/~cpham 

#ifndef __COORDNODE_H
#define __COORDNODE_H

#include <omnetpp.h>
#include "videoSensorNode.h"

#ifdef DISPLAY_SENSOR
#include "display.h"
#endif

// need to keep a scaling factor of 10, the MAXFIELDMETER should be primarily defined and must be equal the the value
// defined in the omnetpp.ini file: SN.field_x = 400 and SN.field_y = 400. A square shaped in required!
#define MAXFIELDUNIT 4000
#define MAXFIELDMETER 400

#define DISPLAY_STATS_INTERVAL 10

#define MIN_OBSTACLE_SIZE 10  // 1m with the scaling factor
#define MAX_OBSTACLE_SIZE 50 // 5m with the scaling factor

class obstacleManager {

public:
	std::list<Segment> obstacles;
#ifdef DISPLAY_SENSOR
	std::list<Segment> obstacles_display;
#endif
	int nbObstacle;

	void init(int, int, int);
	bool intersects(Segment);
	void draw();
};

class coordNode : public cSimpleModule
{ 
  	cMessage *displaystatsEv;
  
protected:
    // The following redefined virtual function holds the algorithm.
    virtual void initialize();
    virtual void handleMessage(cMessage *msg);
	virtual void finish();
public:
	// to get the percentage of coverage
	cOutVector percentageCoverageStatVector;
	cOutVector percentageActiveNodeStatVector;
	cOutVector stddevEnergyLevelStatVector;
	cDoubleHistogram meanPercentageCoversetCoverage;
	cDoubleHistogram meanNumberCoverset;
	cDoubleHistogram meanCaptureRate;
	cDoubleHistogram meanEnergyLevel;

#ifdef CRITICALITY_ADAPTATIVE_MAC
	//std::vector<int> alertNumbersVector;
	//std::vector<int> alertResponseVector;

    int alertResponseArray[200][700];
    int alertNumber[200];
#endif

	int nbrNode;
	int fieldMaxX;
	int fieldMaxY;

    double initialCoverage;
    double roundCoverage;

    noeud *allNodes;
    int nbRegisteredSensorNodeCounter; // count the number of registered sensor nodes

	obstacleManager obstacleList; // store and manage the obstacle

    float distance(Point , Point);
    double percentageCoverage(noeud* , int);
#ifdef SCHEDULE_PROB
	bool probWakeNode(unsigned int, int);
	bool probTestNode(unsigned int, int);
#endif
    double percentageCoverageRound(noeud* , int);
	void fillNodes(Point p, Point v, Point b, Point c, int id, int nbCamera, int cameraIndex, VideoSensorNode* theNode);
	void updateNodesStatus(int id, bool isActive, bool isAlive=true);
    void nodeFinish(int);
	noeud* getNodeInfo(int);
#ifdef CRITICALITY_ADAPTATIVE_MAC
    void countAlertNumber(int id);
	void countAlertResponse(int id, int alertNb);
#endif
};
#endif
