#include <fstream>
#include <assert.h>
#include <stdlib.h>
#include <iostream>
#include <time.h>
#include <math.h>

#include "Point.h"
#include "Segment.h"
#include "Triangle.h"
#include "Polygon.h"
#include "graphic_interface.h"


#define FRAME_LIMIT 1000 / 500 		//Limit the animation to 20 frames per second

struct point {
    int x;
    int y;
};

#define PI        3.141592653589793
// assume 600px means 200m
#define maxFieldX 600
#define maxFieldY 600
#define borderX 20
#define borderY 20

// half the angle of the field of view
static double alpha = PI/10;

// here 600 means 200m (2000 in the OMNET++ simulation), so here 75 means 25m for the FoV
static int Rs = 75;

extern SDL_Surface *screen;  
extern SDL_Event event;

using namespace std;

// Based on the triangle sampling method found on http://mathworld.wolfram.com/TrianglePointPicking.html
// Basically, use x = a1V1+a2V2; where a1 & a2 are between 0 and 1, and V1 & V2 two vector of the triangle
Point sampleTriangle(const Triangle & t) {

	double a1;
	double a2;
	bool found=false;
	Point randomP;

	Point AB = t.v2-t.v1;
	//cout << "** Point AB(" << AB.get_x() << "," << AB.get_y() << ")" << endl;

	Point AC = t.v3-t.v1;
	//cout << "** Point AC(" << AC.get_x() << "," << AC.get_y() << ")" << endl;

	while (!found) {
		a1=(double)rand() / (double)RAND_MAX;
		a2=(double)rand() / (double)RAND_MAX;
		//cout << a1 << ", " << a2 << ", "; 
		randomP = a1*AB+a2*AC;
		randomP = randomP + t.v1;
		//cout << "(" << randomP.get_x() << "," << randomP.get_y() << ")" << endl;
		if (randomP.is_inside(t)) { 
			found=true;
			//cout << "found" << endl;
		}
//		else cout << "not found - retry" << endl;
	}

	return (randomP);
}

float myDistance(point& M, point& N) {
    return sqrt((M.x - N.x)*(M.x - N.x) + (M.y - N.y)*(M.y - N.y));
}

bool isCoveredBy(point& P1, point& P, point& v) {
    float D = myDistance(P1,P);
    double PP1x = P1.x - P.x;
    double PP1y = P1.y - P.y;
    double Pvx = v.x - P.x;
    double Pvy = v.y - P.y;

    double produitScalaire = PP1x*Pvx + PP1y*Pvy;

    if ((D<=Rs)&&(produitScalaire>=D*cos(alpha)))
        return true;
    else
        return false;
}

int main (int argc, char *argv[]) {

	Point V2(137.0,232.0);
	Point V3(216.0,91.0);
	Point V1(40.0,40.0);
	Point P1, P2, P3;
	Point randomP;
	Polygon theField;
	Triangle myTriangle;
	point p, v, b, c, g, x;
	int quit = 0;
	int anim = 0;	
	unsigned timer;
	
	int nbNodes = atoi(argv[1]);
	Rs = atoi(argv[2]);

	srand (time(NULL));

	//srand (50);

	cout << "Start" << endl; 

	if (SDL_Init(SDL_INIT_VIDEO) !=0)  
		return 1;
   	screen = SDL_SetVideoMode(GRAPHICS_WIDTH, GRAPHICS_HEIGHT, BITDEPTH, SDL_SWSURFACE);
	
	if (screen == NULL)  
		return 1;

	SDL_FillRect(screen, NULL, 0xFFFFFFFF);

	P1 = Point(Rs+10,Rs+10);
	theField.push_back(P1);
	P1 = Point(Rs+10+maxFieldX,Rs+10);
	theField.push_back(P1);
	P1 = Point(Rs+10+maxFieldX,Rs+10+maxFieldY);
	theField.push_back(P1);
	P1 = Point(Rs+10,Rs+10+maxFieldY);
	theField.push_back(P1);
	
	theField.draw();

	nbNodes=1;
	
	for (int i=0; i<nbNodes; i++) { 
	
		p.x=100;
		p.y=100;

		// cout << "P(" << p.x << "," << p.y << ")" << endl;

		double angle = ((double)rand() / (double)RAND_MAX) * PI;

		angle = 0.8;

		//cout << "angle " << angle << endl;

		v.x=Rs*cos(angle)+p.x;
		v.y=Rs*sin(angle)+p.y;

		// find point b, c & g for each sensor node
		double h = Rs*sin(alpha);
		double d = Rs*cos(alpha);

		b.x = v.x + h*(v.y - p.y)/d;
		b.y = v.y - h*(v.x - p.x)/d;

		c.x = v.x - h*(v.y - p.y)/d;
		c.y = v.y + h*(v.x - p.x)/d;

		g.x = (p.x + b.x + c.x)/3;
		g.y = (p.y + b.y + c.y)/3;
	
		P1 = Point(p.x,p.y);
		P2 = Point(b.x,b.y);
		P3 = Point(c.x,c.y); 
		myTriangle=Triangle(P1,P2,P3);
		myTriangle.draw();

	
/*
	Point P (40.0, 38.0);

	if (P.is_inside(myTriangle)) cout << "Point P(" << P.get_x() << "," << P.get_y() 
		<< ") is inside the triangle" << endl;
	else
		cout << "Point P(" << P.get_x() << "," << P.get_y() << ") is outside the triangle" << endl;

	Point Barycentre = (V1+V2+V3)/3;

	cout << "Point G(" << Barycentre.get_x() << "," << Barycentre.get_y() << ")" << endl;

	for (int i=0; i<10; i++) {
		randomP = sampleTriangle(myTriangle);
		cout << "Point randomP(" << randomP.get_x() << "," << randomP.get_y() << ")" << endl;
	}	
*/
	}

	if (SDL_Flip(screen)==-1) 
			return 1;
	
	while(!quit)
	{
		Polygon theSquare;
		//Get the start time at the begining of each loop
		timer = SDL_GetTicks();	

		x.x=rand()%GRAPHICS_WIDTH;
		x.y=rand()%GRAPHICS_HEIGHT;

		randomP=Point(x.x,x.y);
		randomP.draw();

		P1 = Point(x.x-5,x.y-5);
		theSquare.push_back(P1);
		P1 = Point(x.x+5,x.y-5);
		theSquare.push_back(P1);
		P1 = Point(x.x+5,x.y+5);
		theSquare.push_back(P1);
		P1 = Point(x.x-5,x.y+5);
		theSquare.push_back(P1);	
		theSquare.draw();

		cout << "------------------------------" << endl;

		if (isCoveredBy(x,p,v))
			cout << "point X(" << x.x << "," << x.y << ") is covered" << endl; 
		else
			cout << "point X(" << x.x << "," << x.y << ") is not covered" << endl; 

		if (randomP.is_inside(myTriangle))
			cout << "Point randomP(" << randomP.get_x() << "," << randomP.get_y() << ") is covered" << endl;
		else
			cout << "Point randomP(" << randomP.get_x() << "," << randomP.get_y() << ") is not covered" << endl;
	
		//randomP = sampleTriangle(myTriangle);
		//randomP.draw();
		//cout << "Point randomP(" << randomP.get_x() << "," << randomP.get_y() << ")" << endl;
			

        // Flip the stuff drawn into view
		if (SDL_Flip(screen)==-1) 
			return 1;
		
		//This code checks for a keypress, which signals to end the program
		while(SDL_PollEvent(&event))
                     if(event.type==SDL_KEYDOWN)
                         quit=1;
		
		//Increment  animation
		anim++; 	

		//This delays the program so that it does not run too fast
		//timer = SDL_GetTicks()-timer;
		//if(timer < FRAME_LIMIT) SDL_Delay(FRAME_LIMIT-timer);

		SDL_Delay(1000);
	} 


    SDL_Quit();
	cout << "number of points" << anim << endl;
	return 0;
}
