#include <fstream>
#include <assert.h>
#include <stdlib.h>
#include <iostream>
#include <time.h>
#include <math.h>

#include "Point.h"
#include "Segment.h"
#include "Triangle.h"
#include "Polygon.h"
#include "graphic_interface.h"


#define FRAME_LIMIT 1000 / 500 		//Limit the animation to 20 frames per second

#define PI        3.141592653589793
// assume in meters with a scaling factor of 3 for visibility
#define maxFieldX 75*3
#define maxFieldY 75*3

// half the angle of the field of view
static double alpha = PI/10;

// means 25m for the FOV
static int rs = 25*3;
static int Rs;

extern SDL_Surface *screen;  
extern SDL_Event event;

using namespace std;

// Based on the triangle sampling method found on http://mathworld.wolfram.com/TrianglePointPicking.html
// Basically, use x = a1V1+a2V2; where a1 & a2 are between 0 and 1, and V1 & V2 two vector of the triangle
Point sampleTriangle(const Triangle & t) {

	double a1;
	double a2;
	bool found=false;
	Point randomP;

	Point AB = t.v2-t.v1;
	//cout << "** Point AB(" << AB.get_x() << "," << AB.get_y() << ")" << endl;

	Point AC = t.v3-t.v1;
	//cout << "** Point AC(" << AC.get_x() << "," << AC.get_y() << ")" << endl;

	while (!found) {
		a1=(double)rand() / (double)RAND_MAX;
		a2=(double)rand() / (double)RAND_MAX;
		//cout << a1 << ", " << a2 << ", "; 
		randomP = a1*AB+a2*AC;
		randomP = randomP + t.v1;
		//cout << "(" << randomP.get_x() << "," << randomP.get_y() << ")" << endl;
		if (randomP.is_inside(t)) { 
			found=true;
			//cout << "found" << endl;
		}
//		else cout << "not found - retry" << endl;
	}
	return (randomP);
}

float myDistance(Point M, Point N) {
    return sqrt((M.x - N.x)*(M.x - N.x) + (M.y - N.y)*(M.y - N.y));
}


void findTriangleCoord(Point p, Point v, Point& b, Point& c) {
    double h = Rs*sin(alpha);
    double d = Rs*cos(alpha);

    b.x = v.x + h*(v.y - p.y)/d;
    b.y = v.y - h*(v.x - p.x)/d;

    c.x = v.x - h*(v.y - p.y)/d;
    c.y = v.y + h*(v.x - p.x)/d;
}

int main (int argc, char *argv[]) {

	Point P1;
	Point p,v,b,c,g;
	Point randomP;
	Triangle myTriangle;
	Polygon theField;
	SDL_Rect myField;

	int quit = 0;
	int anim = 0;	
	unsigned timer;
	
	int nbNodes = atoi(argv[1]);
	if (argc > 2) 
		Rs = atoi(argv[2]);
	else Rs=rs;

	srand (time(NULL));

	//srand (50);

	cout << "Start" << endl; 

	if (SDL_Init(SDL_INIT_VIDEO) !=0)  
		return 1;
   	screen = SDL_SetVideoMode(GRAPHICS_WIDTH, GRAPHICS_HEIGHT, BITDEPTH, SDL_SWSURFACE);
	
	if (screen == NULL)  
		return 1;

	SDL_FillRect(screen, NULL, 0xFFFFFFFF);
	
	/*
	myField.x=borderX;
	myField.y=borderY;
	myField.w=maxFieldX;
	myField.h=maxFieldY;
	SDL_SetClipRect(screen, NULL);
	*/

	P1 = Point(Rs+10,Rs+10);
	theField.push_back(P1);
	P1 = Point(Rs+10+maxFieldX,Rs+10);
	theField.push_back(P1);
	P1 = Point(Rs+10+maxFieldX,Rs+10+maxFieldY);
	theField.push_back(P1);
	P1 = Point(Rs+10,Rs+10+maxFieldY);
	theField.push_back(P1);
	
	theField.draw();
	
	for (int i=0; i<nbNodes; i++) { 
	
		p.x=rand()%maxFieldX+Rs+10;
		p.y=rand()%maxFieldY+Rs+10;

		// cout << "P(" << p.x << "," << p.y << ")" << endl;
		// old method, not good!
    	if (rand()%100 < 50){
        	v.x = rand()%(110) + p.x;
       		v.y = p.y + sqrt(Rs*cos(alpha)*Rs*cos(alpha) - ((p.x - v.x)*(p.x - v.x)));
    	}
    	else 
    	{     
        	v.x = p.x - rand()%(110);
        	v.y = p.y - sqrt(Rs*cos(alpha)*Rs*cos(alpha) - ((p.x - v.x)*(p.x - v.x)));
    	}
		
		// new method!
		double angle = ((double)rand() / (double)RAND_MAX) * 2*PI;

		//cout << "angle " << angle << endl;

		v.x=Rs*cos(angle)+p.x;
		v.y=Rs*sin(angle)+p.y;

		// find point b, c & g for each sensor node
		findTriangleCoord(p, v, b, c);

		g.x = (p.x + b.x + c.x)/3;
		g.y = (p.y + b.y + c.y)/3;
	
		myTriangle=Triangle(p,b,c);
		myTriangle.draw();
	}

	if (SDL_Flip(screen)==-1) 
			return 1;
	
	//This is the main loop that runs until the program ends
	while(!quit)
	{
		//Get the start time at the begining of each loop
		timer = SDL_GetTicks();	
	
		randomP = sampleTriangle(myTriangle);
		randomP.draw();
		//cout << "Point randomP(" << randomP.get_x() << "," << randomP.get_y() << ")" << endl;

        // Flip the stuff drawn into view
		if (SDL_Flip(screen)==-1) 
			return 1;
		
		//This code checks for a keypress, which signals to end the program
		while(SDL_PollEvent(&event))
                     if(event.type==SDL_KEYDOWN)
                         quit=1;
		
		//Increment  animation
		anim++; 	

		//This delays the program so that it does not run too fast
		timer = SDL_GetTicks()-timer;
		if(timer < FRAME_LIMIT) SDL_Delay(FRAME_LIMIT-timer);
	} 

	SDL_Quit();
	cout << "number of points" << anim << endl;
	return 0;
}
