// C. Pham, Congduc.Pham@univ-pau.fr, www.univ-pau.fr/~cpham 

#include <cstdlib>
#include <omnetpp.h>
#include "intrusion.h"
#include "coordNode.h"
#include "videoSensorNode.h"

#ifdef FUNCTION_TRACE_DEBUG
#include "mydebug.h"
#endif

// The module class needs to be registered with OMNeT++
Define_Module(intrusionModule);

void intrusionModule::initialize()
{
	cModule *coordNodeModule = getParentModule()->getSubmodule("coordinator");
    CoordNode = check_and_cast<coordNode *>(coordNodeModule);

    cDisplayString &nodes = getDisplayString();
    guiX = std::atoi (nodes.getTagArg("p",0));
    guiY = std::atoi (nodes.getTagArg("p",1));
	// first time of intrusion
	this->activatedAt = par("activatedAt");
	// starts a new intrusion
	this->startNewIntrusionPosition = true;
	this->intrusionTotalViewCount=0;	
	this->intrusionViewCount=0;
	this->intrusionMissed=0;
	this->intrusionOn=false;
	this->enableHit=true;

#ifdef WITH_INTRUSION
	stealthTimeStatVector.setName("stealthTime");

#ifdef INTRUSION_POLYGON
	nbPolyPointOriginal.setName("nbPolyPointOriginal");
	nbMinPolyPoint.setName("nbMinPolyPoint");
	nbMaxPolyPoint.setName("nbMaxPolyPoint");
#endif
	// change the number of intrusions here
	this->nbIntrusion = 10;

#ifdef INTRUSION_SUCC_SCANLINE
	// starts at the left side
	guiX = 0;
	guiY = 10;
	this->p.x = guiX+MAX_DOV_INTERNAL;
	this->p.y = guiY+MAX_DOV_INTERNAL;
#endif

    this->intrusionEv = new cMessage("intrusion");
	scheduleAt(simTime()+this->activatedAt, this->intrusionEv);
#endif
}

void intrusionModule::handleMessage(cMessage *msg)
{
#ifdef WITH_INTRUSION
	// an intrusion occurs
	if (strcmp("intrusion", msg->getName())== 0) {
		this->intrusionOn=true;

		if (this->startNewIntrusionPosition) {
			recordScalar("info.intrusion.nb_intrusion_remaining", this->nbIntrusion);
			recordScalar("info.intrusion.start_time", simTime());
#ifdef INTRUSION_SUCC_SCANLINE
			// starts a new trajectory from the left side
			this->guiX = 0;
			this->guiY += 60;
			this->p.x = guiX+MAX_DOV_INTERNAL;
		 	this->p.y += 60;			
#endif
#ifdef INTRUSION_RAND_POSITION
			// generate a random starting position
			// for x, not too close from the right border
			this->guiX = uniform(10, CoordNode->fieldMaxX-200);
			this->guiY = uniform(10, CoordNode->fieldMaxY-20);
			this->p.x = guiX+MAX_DOV_INTERNAL;
			this->p.y = guiY+MAX_DOV_INTERNAL;
#ifdef INTRUSION_POLYGON
			// if we want a polygon, point p is the upper-left point
			createIntrusionPolygon();
#endif

#ifdef DEBUG_OUTPUT_LEVEL0
			ev << simTime() << " NEW MOVE(" <<  this->p.x << "," <<
				this->p.y << ")," << endl;
#endif
#endif
			// this is the time at which the intrusion appears
			this->activatedAt=SIMTIME_DBL(simTime());
			this->enableHit=true;
			this->startNewIntrusionPosition=false;

			// reset the intrusion count for the next detection
			this->intrusionViewCount=0;

			// comment the following line if number of intrusion is unlimited
#ifndef INTRUSION_RAND_POSITION
			this->nbIntrusion--;
#endif
			if (ev.isGUI())
				updateDisplayProgress();
		}
		else
			if (this->p.x < (CoordNode->fieldMaxX+MAX_DOV_INTERNAL))
				intrusionProgress();
			else {
				if (this->intrusionViewCount==0)
					this->intrusionMissed++;
				// start a new position as the intruder has move outside the area of interest
				this->startNewIntrusionPosition=true;
			}
			
		if (this->nbIntrusion)
			// schedule next move 1s in the future
			scheduleAt(simTime()+1.0, this->intrusionEv);
		else {
			this->intrusionOn=false;
			recordScalar("info.intrusion.end_time", simTime());
		}
    }
#endif
}

void intrusionModule::createIntrusionPolygon()
{
#ifdef WITH_INTRUSION
#ifdef INTRUSION_POLYGON
	// clear the polygon first
	myPolygon.vertices.clear();

	// point p is the upper-left point
	// p---a---b
	// -       -
	// e       c
	// -       -
	// f---g---d
	
	// p
	Point a=this->p;
	myPolygon.push_back(a);	
	// a
	// apply a scaling factor as the coordinate system used is not in meters
	a = a + Point(POLYGON_INTERPOINT_DISTANCE_H*MAXFIELDUNIT/MAXFIELDMETER,0);
	myPolygon.push_back(a);
	//b
	a = a + Point(POLYGON_INTERPOINT_DISTANCE_H*MAXFIELDUNIT/MAXFIELDMETER,0);
	myPolygon.push_back(a);
	// c
	a = a + Point(0,POLYGON_INTERPOINT_DISTANCE_V*MAXFIELDUNIT/MAXFIELDMETER);
	myPolygon.push_back(a);
	// d	
	a = a + Point(0,POLYGON_INTERPOINT_DISTANCE_V*MAXFIELDUNIT/MAXFIELDMETER);
	myPolygon.push_back(a);
	// e
	a=this->p;
	a = a + Point(0,POLYGON_INTERPOINT_DISTANCE_V*MAXFIELDUNIT/MAXFIELDMETER);
	myPolygon.push_back(a);
	// f
	a = a + Point(0,POLYGON_INTERPOINT_DISTANCE_V*MAXFIELDUNIT/MAXFIELDMETER);
	myPolygon.push_back(a);	
	// g
	a = a + Point(POLYGON_INTERPOINT_DISTANCE_H*MAXFIELDUNIT/MAXFIELDMETER,0);
	myPolygon.push_back(a);	
#endif
#endif
}

void intrusionModule::updateDisplayProgress()
{
#ifdef WITH_INTRUSION
	char s[10];
	cDisplayString &nodes = getDisplayString();
	
	sprintf(s,"%d",guiX); 
	nodes.setTagArg("p",0,s);
	sprintf(s,"%d",guiY); 
	nodes.setTagArg("p",1,s);

	sprintf(s, "%d(%d)", intrusionViewCount, nbIntrusion);
	nodes.setTagArg("t",0,s);
#endif
}

// move the intrusion Point or Polygon. 
void intrusionModule::intrusionProgress()
{
#ifdef WITH_INTRUSION
	// move the intrusion point, apply a scaling factor as the coordinate system used is not in meters
	p.x = p.x + INTRUSION_VELOCITY*MAXFIELDUNIT/MAXFIELDMETER;
	// update position in the OMNET++ GUI
	guiX = guiX + INTRUSION_VELOCITY*MAXFIELDUNIT/MAXFIELDMETER;

#ifdef INTRUSION_POLYGON
	// update the intrusion polygon
	createIntrusionPolygon();
#endif

#ifdef DEBUG_OUTPUT_LEVEL0
#ifdef INTRUSION_RAND_POSITION
    ev << "\t" << simTime() << "\t M(" <<  p.x << "," <<  
		p.y << ")," << endl;  
#endif
#ifdef INTRUSION_SUCC_SCANLINE
    ev << "**INTRUSION** " << simTime() << " at (" <<  p.x << "," <<  
		p.y << ")" << endl; 
#endif
#endif  
	if (ev.isGUI())
		updateDisplayProgress();
#endif
}

void intrusionModule::intrusionHit(int id)
{
#ifdef WITH_INTRUSION

	double stealthTime;
	
	// don't take hit anymore
	if (!enableHit)
		return;

#ifdef INTRUSION_SUCC_SCANLINE
	// with the scan line model, we only record the time of the first intrusion detection
	if (intrusionTotalViewCount==0) {
		intrusionFirstViewTime=SIMTIME_DBL(simTime());
		stealthTime = intrusionFirstViewTime-activatedAt;
	}
#endif

#ifdef INTRUSION_STEALTH_TIME
	// when we are concerned by the stealth time, we record the stealth time each time a sensor sees the intrusion
	// in case the intrusion is a polygon, then seeing one point of the polygon is enough
	intrusionFirstViewTime=SIMTIME_DBL(simTime());
	stealthTime=intrusionFirstViewTime-activatedAt;

	// collect in the cDoubleHistogram class
	//stealthTimeHistogram.collect(stealthTime);

	// record each sample in the .vec file for time plotting
	stealthTimeStatVector.record(stealthTime);

	// indicate that we start a new intrusion at a new random position
	startNewIntrusionPosition=true;
#ifdef DEBUG_OUTPUT_LEVEL0
    ev << "STOP MOVING" << endl;  
#endif
#endif

	intrusionTotalViewCount++;
	intrusionViewCount++;
	
	char s[10];
	sprintf(s,"%.2f(%d)", stealthTime, id);
	if (ev.isGUI())
		bubble(s); 
	// other sensors can not hit anymore until next move otherwise the stealth time is not correct
	enableHit=false;
#endif
}

int intrusionModule::isPolygonInside(Triangle fov)
{
#ifdef WITH_INTRUSION
#ifdef INTRUSION_POLYGON
	int j=0;
	Point p;
	// find how many points are inside the FoV
	for (Polygon::iterator i=myPolygon.begin(); i!=myPolygon.begin()-1; i++, j++) {
		p=*i;
		if (p.is_inside(fov)) {
			// records which point of the polygon is seen
			polygonPointTable[j]++;
		}
	}
	int count=0;
	for (int i=0; i<POLYGON_NB_POINT; i++)
		if (polygonPointTable[i]) 
			count++;
	// actually returns the number of points of the intrusion polygon that have been seen so far
	return count;
#endif
#endif
}

void intrusionModule::clearPolygonPointTable()
{
#ifdef WITH_INTRUSION
#ifdef INTRUSION_POLYGON
	memset(polygonPointTable,0,POLYGON_NB_POINT*sizeof(int));
#endif
#endif
}

void intrusionModule::finish()
{
#ifdef FUNCTION_TRACE_DEBUG
	TRACE_DEBUG_IN("finishSpecific()");
#endif

#ifdef WITH_INTRUSION
	recordScalar("stat.intrusion.intrusionTotalViewCount", intrusionTotalViewCount);
	recordScalar("stat.intrusion.intrusionMissed", intrusionMissed);
	//recordScalar("stat.intrusion.intrusionStealthTime", intrusionFirstViewTime-activatedAt);
#ifdef DEBUG_OUTPUT_LEVEL0
	ev << "intrusionTotalViewCount=" << intrusionTotalViewCount << endl;
	ev << "intrusionMissed=" << intrusionMissed << endl;
	//ev << "intrusionStealthTime " << intrusionFirstViewTime-activatedAt << endl;
#endif		

#ifdef INTRUSION_STEALTH_TIME
	//stealthTimeHistogram.recordAs("stealthTime");
#endif

	cancelAndDelete(intrusionEv);
#endif

#ifdef FUNCTION_TRACE_DEBUG
	TRACE_DEBUG_OUT("finishSpecific()");
#endif
}


