// C. Pham, Congduc.Pham@univ-pau.fr, www.univ-pau.fr/~cpham 

#ifndef __INTRUSION_H
#define __INTRUSION_H

#include <omnetpp.h>

#include "geometry/Point.h"
#include "geometry/Segment.h"
#include "geometry/Triangle.h"
#include "geometry/Polygon.h"

#include "videoModelDefine.h"

// velocity in meter/s
#define INTRUSION_VELOCITY 5
#define POLYGON_NB_POINT 8
#define POLYGON_INTERPOINT_DISTANCE_H 4
#define POLYGON_INTERPOINT_DISTANCE_V 2

class coordNode;

class intrusionModule : public cSimpleModule
{
	cMessage *intrusionEv;
	coordNode *CoordNode;
  protected:
    // The following redefined virtual function holds the algorithm.
    virtual void initialize();
    virtual void handleMessage(cMessage *msg);
	virtual void finish();
  public:
	Point p;
	Polygon myPolygon;
	int myPolygonNbPoint;
	int guiX;
	int guiY;

	bool intrusionOn;
	bool enableHit;
	int intrusionViewCount;
	int intrusionTotalViewCount;
	int intrusionMissed;
	int nbIntrusion;
	bool startNewIntrusionPosition;
	double activatedAt;
	double intrusionFirstViewTime;
	// records the stealh time to get statistics
	cDoubleHistogram stealthTimeHistogram;
	cOutVector stealthTimeStatVector;
#ifdef INTRUSION_POLYGON
	int polygonPointTable[POLYGON_NB_POINT];
	// the number of points of the intrusion polygon seen by the original sensor
	cOutVector nbPolyPointOriginal;
	// the min number of points of the intrusion polygon seen by the cover sets
	cOutVector nbMinPolyPoint;
	// the max number of points of the intrusion polygon seen by the cover sets
	cOutVector nbMaxPolyPoint;
#endif

	void updateDisplayProgress();
	void intrusionProgress();
	void intrusionHit(int);
	void createIntrusionPolygon();
	int isPolygonInside(Triangle);
	void clearPolygonPointTable();
};

#endif
