#include <iostream>
#include <math.h>
#include <stdlib.h>
#include <time.h>

using std::cout;
using std::cin;
using std::endl;

// the size of the field, could say that 750 means 75m for instance
int MaxX = 750;
int MaxY = 750;

// communication range, as unit as above, 300 means 30m
const double Rc = 300;

const int maxDefinedCoverSetNumber = 12;

struct point {
  int x;
  int y;
  int gateSize;
  int *neighbors;
  int in;
  double load;
};

float distance (point&, point&);
void coordon (point&);

float distance (point& M, point& N)
{
  return sqrt((M.x - N.x)*(M.x - N.x) + (M.y - N.y)*(M.y - N.y));
}

void coordon(point& P)
{
  P.x = rand()% MaxX;
  P.y = rand()% MaxY;
}

int getMaxDefinedCoverSetNumber()
{
    return maxDefinedCoverSetNumber;
}

int main(int argc, char *argv[]){
  	srand (time(NULL));
  	point *nodes;
  	int nbNodes;
	
  	double sum = 0;

	if (argc < 2) {
		cout << "You must specified the number of nodes. Ex: generate 75" << endl;
		exit(1);
	}
	else
		nbNodes = atoi(argv[1]);

	if (argc==2) {
		cout << "// Use default size = 75m x 75m." << endl;
	}	

	if (argc==3) {
		cout << "// You must also specified the size of the field. Ex: generate 75 75 75" << endl;
		cout << "// Use default size = 75m x 75m." << endl;
	}
	
	if (argc==4) {
		MaxX = atoi(argv[2]);
		MaxX = MaxX*10;
		MaxY = atoi(argv[3]);
		MaxY = MaxY*10;
	}	

  	nodes = new point[nbNodes];

  	for (int i=0; i<nbNodes; i++) {  
  		// set a random position in the field
    	coordon(nodes[i]); 
    	nodes[i].gateSize=0;
	
		if (i==0) nodes[i].load = rand()%nbNodes;

    	while (nodes[i].load == 0) 
    		nodes[i].load =  rand()%nbNodes;
    	sum += nodes[i].load;
  	}

	//normalize between 0 and 1
  	for (int i=0; i<nbNodes; i++) {
   		nodes[i].load = nodes[i].load*nbNodes/sum;
  	}

#ifdef GENERATE_POS
	// generate only the node's position
  	for (int i=0; i<nbNodes; i++) 
		cout << "SN.node[" << i << "].xCoor = "  <<  nodes[i].x/10 << "\n" <<
				"SN.node[" << i << "].yCoor = "  <<  nodes[i].y/10 << "\n" <<
				"SN.node[" << i << "].zCoor = 0" << "\n" << endl;

#else

	// Generate the .ned file	

	cout << "simple VideoSensorNode \n" <<
		 	"{ \n" << 
    			"\t parameters:\n";
  			    			
	cout <<			"\t\t // specific to our model" << endl <<
					"\t\t double minCaptureRate;\n"<<
					"\t\t double maxCaptureRate;\n"<<
					"\t\t double staticCaptureRate;\n"<<
					"\t\t double aov;\n"<<
					"\t\t double dov;\n"<<
					"\t\t double maxBatteryLevel;\n"<<
					"\t\t double energyPerCapture;\n"<<
					"\t\t double criticalityLevel;\n"<<
					"\t\t double minCriticalityLevel;\n"<<
					"\t\t double maxCriticalityLevelPeriod;\n"<<
					"\t\t double maxDefinedCoverSetNumber;\n"<<
					"\t\t double forceNumberOfCoversets;\n"<<
					"\t\t bool isMobile;\n"<<
					"\t\t double isMobileProb;\n"<<
					"\t\t bool isCamRotatable;\n"<<
					"\t\t double isCamRotatableProb;\n"<<
					"\t\t double maxMobilitySpeed;\n"<<
					"\t\t double maxCamRotationTime;\n"<<
					"\t\t double maxCamRotationCount;\n"<<
    				"\t\t double load;\n"<<
    			"\t gates: \n";			
    cout << 		"\t\t input in[]; \n"<<
    				"\t\t output out[]; \n";    				
    cout << "}" << endl << endl << endl;

	cout << "simple coordNode \n" << 
		 	"{ \n" << 
				"\t parameters:\n"<<
					"\t\t double fieldMaxX;\n"<<
					"\t\t double fieldMaxY;\n"<<
					"\t\t double nodesNb;\n"<<
			"}" << endl << endl;

	cout << "simple intrusionModule \n" << 
		  	"{ \n" << 
				"\t parameters:\n"<<
					"\t\t double activatedAt;\n"<<
			"}" << endl << endl;

	// the network
  	cout << "network SN \n"<<
		  	"{ \n" << 
    			"\t parameters: \n"<<
    				"\t\t double field_x;\n"<<
    				"\t\t double field_y;\n"<<
    				"\t\t double field_z;\n"<<
    				"\t\t double numNodes;\n"<<    					
    			"\t submodules: \n" << endl;

	cout <<			"\t\tcoordinator: coordNode {\n"<<
						"\t\t\t parameters: \n"<<
						    "\t\t\t\t fieldMaxX = field_x*10;\n"<<
						    "\t\t\t\t fieldMaxY = field_y*10;\n"<<
							"\t\t\t\t nodesNb = numNodes; \n"<<
							"\t\t\t\t @display(\"i=block/circle_vs,blue;p=0,0\");" << endl << 
					"\t\t}" << endl;

	cout <<			"\t\tintrusion: intrusionModule {\n"<<
						"\t\t\t parameters: \n"<<
							"\t\t\t\t activatedAt = 10.0; \n"<<
						"\t\t\t\t @display(\"i=block/user_s,blue;p=0," << MaxY << "\");" << endl << 
					"\t\t}"<< endl;


	// set communication channels count if nodes are in range of each other
  	for(int i=0; i<nbNodes; i++){
    	for(int j=i+1; j<nbNodes; j++){
      		if ((distance(nodes[i], nodes[j]) <= Rc)&&(i!=j)){
				nodes[i].gateSize++;
				nodes[j].gateSize++;
      		}
    	}
  	}
	// generate each sensor node
  	for	(int i=0; i<nbNodes; i++){
    	nodes[i].in = 0;
    	nodes[i].neighbors =  new int[nodes[i].gateSize];
    	cout << "\t\tnode" << i << ": VideoSensorNode {\n" <<
      				"\t\t\t parameters:\n" <<
						"\t\t\t\t // most parameters are set with the .ini file \n" <<
      					"\t\t\t\t load = " << nodes[i].load << "; \n" <<
      					"\t\t\t\t @display(\"i=block/circle_vs,cyan;p=" << 
      					nodes[i].x <<","<<nodes[i].y<<"\");" << endl <<
      				"\t\t\t gates: \n" <<
						"\t\t\t\t in[" << nodes[i].gateSize << "];\n" <<
						"\t\t\t\t out[" << nodes[i].gateSize << "]; \n" <<
				"\t\t}"<< endl;
  	}

  	for (int i=0; i<nbNodes; i++) {
  		int k=0;
    	for(int j=0; j<nbNodes; j++){
      		if((distance(nodes[i], nodes[j]) <= Rc)&&(i!=j)){
				if (k>nodes[i].gateSize){cout << "errrrrroooooooooooorrrrrrrrr" << endl;}
			nodes[i].neighbors[k]=j;
			k++;
      		}
    	}
  	}

  	cout << "\n" << "\t\tconnections:" << endl;
	// set connections
 	for(int i=0; i<nbNodes; i++) {
   		for(int j=0; j<nodes[i].gateSize; j++){

     		cout<<"\t\t\t node"<<i<<".out["<<j<<"] --> { delay=0.1ms; datarate=0.0; @display(\"ls=black,0\"); } --> node"<<
     			nodes[i].neighbors[j]<<".in["<<nodes[nodes[i].neighbors[j]].in<<"];"<<endl;

     		nodes[nodes[i].neighbors[j]].in++;
   		}
 	}

 	cout << "}" << endl;
#endif

  	return 0;
}





